unit CTS.Entities.SEQ.CrAt_UpAt;

interface

uses
  //SysUtils,
  //Generics.Collections,

  System.DateUtils,                                   // for TDateTime.NowUTC
  Aurelius.Mapping.Explorer, Aurelius.Events.Manager, // for TUpdatingArgs

  Aurelius.Mapping.Attributes,
  //Aurelius.Types.Blob,
  //Aurelius.Types.DynamicProperties,
  Aurelius.Types.Nullable;

type
  TSeqEntity = class;

//***********************************************
// https://doc.tmssoftware.com/biz/aurelius/guide/events.html#using-attributes
// By default, Delphi doesn't generate RTTI for non-published methods!
// it is then MANDATORY to add directive below!
//***********************************************
{$RTTI EXPLICIT METHODS([vcPrivate, vcProtected, vcPublic, vcPublished]) }
  [AbstractEntity]
  TSeqEntity = class abstract
  strict private
    [Column('createdAt', [TColumnProp.NoUpdate])]
    FcreatedAt: Nullable<TDateTime>;

    [Column('updatedAt', [])]
    FupdatedAt: Nullable<TDateTime>;
  protected
    [OnInserting]     procedure OnInserting;
    [OnUpdating]      procedure OnUpdating(Args: TUpdatingArgs);
  public
    property createdAt: Nullable<TDateTime> read FcreatedAt write FcreatedAt;
    property updatedAt: Nullable<TDateTime> read FupdatedAt write FupdatedAt;
  end;

implementation

{ TSequelizeEntity }

procedure TSeqEntity.OnInserting;
begin
 if FcreatedAt.ValueOrDefault = 0 then
    FcreatedAt := TDateTime.NowUTC;
  if FupdatedAt.ValueOrDefault = 0 then
    FupdatedAt := FcreatedAt;
end;

procedure TSeqEntity.OnUpdating(Args: TUpdatingArgs);
begin
  FupdatedAt := TDateTime.NowUTC;
  if (FupdatedAt < FcreatedAt)  then
    FupdatedAt := FcreatedAt;
  Args.RecalculateState := True;
end;

end.
